#version 150

#define MAX_NUM_LIGHTS 3

// -- TYPES ----------------------------------------------------------------------------------------------------------//

struct Light {
	vec3 position;	// in world coordinates

	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	
	float constantAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;
};
struct Material {
	vec4  ambient;
	vec4  diffuse; 
	vec4  specular;
	float shininess;
};

// -- UNIFORMS -------------------------------------------------------------------------------------------------------//

// lighting
uniform Light[MAX_NUM_LIGHTS]  lights;
uniform int                    usedLights;
uniform vec3                   cameraPosition;

uniform Material material;

// texture
uniform sampler2D textureSampler;
uniform sampler1D celShade;

// -- ATTRIBUTES -----------------------------------------------------------------------------------------------------//

in  vec3  fragmentLightDirections[MAX_NUM_LIGHTS];
in  float fragmentAttenuations[MAX_NUM_LIGHTS];
in  vec3  fragmentViewVector;
in vec3  fragmentNormal;
in vec2  fragmentTexCoord;

out vec4 fragmentColor;

// helper functions
vec4 doAmbient( int index );
vec4 doDiffuse( int index );
vec4 doSpecular( int index );

vec4 celShadeDiffuse( vec4 diffuse ); 

void main()
{
	// do lighting
	vec4 lightColor = vec4( 0, 0, 0, 0 );
	vec4 ambient	= vec4( 0, 0, 0, 0 );
	vec4 diffuse	= vec4( 0, 0, 0, 0 );
	vec4 specular	= vec4( 0, 0, 0, 0 );
	
	for( int i = 0; i < usedLights; i++ )
	{
		ambient += doAmbient( i );
		diffuse += doDiffuse( i );
		specular += doSpecular( i );
	}
	
	//vec4 celDiffuse = celShadeDiffuse( diffuse );
	
	// texture lookup
	vec4 textureColor = texture( textureSampler, fragmentTexCoord );
	
	// combine colors
	lightColor = ambient + diffuse + specular/1000;
	
	fragmentColor = lightColor * textureColor;
}

vec4 doAmbient( int i )
{
	// ambient lighting
	vec4 ambient = material.ambient * lights[i].ambient;

	return ambient;
}

vec4 doDiffuse( int i )
{
	// diffuse lighting	
	float kDiffuse = max( 0, dot( fragmentNormal, fragmentLightDirections[i] ) );
	
	vec4 celShadeValue = texture( celShade, kDiffuse );
	
	vec4 diffuse = material.diffuse * lights[i].diffuse * celShadeValue * fragmentAttenuations[i];
	
	//vec4 diffuse = material.diffuse * lights[i].diffuse * kDiffuse * fragmentAttenuations[i];
	//vec4 diffuse = material.diffuse * lights[i].diffuse * fragmentAttenuations[i];
	
	return diffuse;
}

vec4 doSpecular( int i )
{
	// specular lighting
	vec3  halfWayVector = normalize( fragmentLightDirections[i] + fragmentViewVector );
	float dotProduct    = max( 0, dot( fragmentNormal, halfWayVector ) );
	float kSpecular     = pow( dotProduct, material.shininess );
	
	vec4 specular = material.specular * lights[i].specular * kSpecular * fragmentAttenuations[i];
	
	return specular;
}

vec4 celShadeDiffuse( vec4 diffuse )
{
	vec3 averageLightDir;
	
	for( int i = 0; i < usedLights; ++i )
		averageLightDir += fragmentLightDirections[i];
		
	averageLightDir = normalize( averageLightDir );
	
	float kDiffuse = max( 0, dot( fragmentNormal, averageLightDir ) );
	
	vec4 celShadeValue = texture( celShade, kDiffuse );
	
	vec4 celDiffuse = diffuse * celShadeValue;
	
	return celDiffuse;
}